VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsThreadPoolTimer"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' Autor: F. Schler (frank@activevb.de)
' Datum: 01/2023

Option Explicit

' ----==== Const ====----
Private Const WindowsSystemThreadingThreadPoolTimer As String = "Windows.System.Threading.ThreadPoolTimer"
Private Const IID_IThreadPoolTimer As String = "{594ebe78-55ea-4a88-a50d-3402ae1f9cf2}"
Private Const IID_IThreadPoolTimerStatics As String = "{1a8a9d02-e482-461b-b8c7-8efad1cce590}"

' ----==== Enums ====----
Private Enum vtb_Interfaces
    
    ' IThreadPoolTimerStatics
    IThreadPoolTimerStatics_CreatePeriodicTimer = 6
    IThreadPoolTimerStatics_CreateTimer = 7
    IThreadPoolTimerStatics_CreatePeriodicTimerWithCompletion = 8
    IThreadPoolTimerStatics_CreateTimerWithCompletion = 9

    ' IThreadPoolTimer
    IThreadPoolTimer_GetPeriod = 6
    IThreadPoolTimer_GetDelay = 7
    IThreadPoolTimer_Cancel = 8
       
End Enum

' ----==== Variablen ====----
Private m_pIThreadPoolTimer As Long
Private m_pIThreadPoolTimerStatics As Long

' ----==== Events ====----
Public Event TimerElapsed(ByVal pTimer As Long)
Public Event TimerDestroyed(ByVal pTimer As Long)

' ----==== Class ====----
Private Sub Class_Initialize()
    If GetActivationFactory(WindowsSystemThreadingThreadPoolTimer, _
                            IID_IThreadPoolTimerStatics, _
                            m_pIThreadPoolTimerStatics) Then
    End If
End Sub

Private Sub Class_Terminate()
    Me.Cancel
    Call ReleaseIfc(m_pIThreadPoolTimer)
    Call ReleaseIfc(m_pIThreadPoolTimerStatics)
End Sub

' ----==== Properties ====----
Public Property Let IThreadPoolTimer(ByVal pIThreadPoolTimer As Long)
    Call ReleaseIfc(m_pIThreadPoolTimer)
    m_pIThreadPoolTimer = pIThreadPoolTimer
End Property

Public Property Get IThreadPoolTimer() As Long
    IThreadPoolTimer = m_pIThreadPoolTimer
End Property

' IThreadPoolTimer
Public Property Get Period() As Currency
    Dim Ret As Currency
    If m_pIThreadPoolTimer <> 0& Then
        Dim value As Currency
        If Invoke(m_pIThreadPoolTimer, _
                  IThreadPoolTimer_GetPeriod, _
                  VarPtr(value)) = S_OK Then
            Ret = value
        End If
    End If
    Period = Ret ' TimeSpan
End Property

Public Property Get Delay() As Currency
    Dim Ret As Currency
    If m_pIThreadPoolTimer <> 0& Then
        Dim value As Currency
        If Invoke(m_pIThreadPoolTimer, _
                  IThreadPoolTimer_GetDelay, _
                  VarPtr(value)) = S_OK Then
            Ret = value
        End If
    End If
    Delay = Ret ' TimeSpan
End Property

' ----==== Functions ====----
' IThreadPoolTimerStatics
Public Function CreatePeriodicTimer(ByVal Period As Currency) As Boolean
    If Period < 1 Then Period = 1
    Dim Ret As Boolean
    If m_pIThreadPoolTimerStatics <> 0& Then
        Dim pIThreadPoolTimer As Long
        If Invoke(m_pIThreadPoolTimerStatics, _
                  IThreadPoolTimerStatics_CreatePeriodicTimer, _
                  modTEH.Create(Me), _
                  Period, _
                  VarPtr(pIThreadPoolTimer)) = S_OK Then
            If pIThreadPoolTimer <> 0& Then
                Ret = True
                Me.IThreadPoolTimer = pIThreadPoolTimer
            End If
        End If
    End If
    CreatePeriodicTimer = Ret
End Function

Public Function CreateTimer(ByVal Delay As Currency) As Boolean
    If Delay < 1 Then Delay = 1
    Dim Ret As Boolean
    If m_pIThreadPoolTimerStatics <> 0& Then
        Dim pIThreadPoolTimer As Long
        If Invoke(m_pIThreadPoolTimerStatics, _
                  IThreadPoolTimerStatics_CreateTimer, _
                  modTEH.Create(Me), _
                  Delay, _
                  VarPtr(pIThreadPoolTimer)) = S_OK Then
            If pIThreadPoolTimer <> 0& Then
                Ret = True
                Me.IThreadPoolTimer = pIThreadPoolTimer
            End If
        End If
    End If
    CreateTimer = Ret
End Function

Public Function CreatePeriodicTimerWithCompletion(ByVal Period As Currency) As Boolean
    If Period < 1 Then Period = 1
    Dim Ret As Boolean
    If m_pIThreadPoolTimerStatics <> 0& Then
        Dim pIThreadPoolTimer As Long
        If Invoke(m_pIThreadPoolTimerStatics, _
                  IThreadPoolTimerStatics_CreatePeriodicTimerWithCompletion, _
                  modTEH.Create(Me), _
                  Period, _
                  modTDH.Create(Me), _
                  VarPtr(pIThreadPoolTimer)) = S_OK Then
            If pIThreadPoolTimer <> 0& Then
                Ret = True
                Me.IThreadPoolTimer = pIThreadPoolTimer
            End If
        End If
    End If
    CreatePeriodicTimerWithCompletion = Ret
End Function

Public Function CreateTimerWithCompletion(ByVal Delay As Currency) As Boolean
    If Delay < 1 Then Delay = 1
    Dim Ret As Boolean
    If m_pIThreadPoolTimerStatics <> 0& Then
        Dim pIThreadPoolTimer As Long
        If Invoke(m_pIThreadPoolTimerStatics, _
                  IThreadPoolTimerStatics_CreateTimerWithCompletion, _
                  modTEH.Create(Me), _
                  Delay, _
                  modTDH.Create(Me), _
                  VarPtr(pIThreadPoolTimer)) = S_OK Then
            If pIThreadPoolTimer <> 0& Then
                Ret = True
                Me.IThreadPoolTimer = pIThreadPoolTimer
            End If
        End If
    End If
    CreateTimerWithCompletion = Ret
End Function

' IThreadPoolTimer
Public Function Cancel() As Boolean
    Dim Ret As Boolean
    If m_pIThreadPoolTimer <> 0& Then
        If Invoke(m_pIThreadPoolTimer, _
                  IThreadPoolTimer_Cancel) = S_OK Then
            Ret = True
        End If
    End If
    Cancel = Ret
End Function

' ----==== Invoke TimerHandler ====----
Public Sub Invoke_TimerElapsed(ByVal pTimer As Long)
    RaiseEvent TimerElapsed(pTimer)
End Sub

Public Sub Invoke_TimerDestroyed(ByVal pTimer As Long)
    RaiseEvent TimerDestroyed(pTimer)
End Sub

' ----==== Invoke Interface ====----
Private Function Invoke(ByVal pInterface As Long, _
                        ByVal vtb As vtb_Interfaces, _
                        ParamArray var()) As Variant
    If pInterface <> 0& Then
        Invoke = OleInvoke(pInterface, vtb, var)
    End If
End Function

